/**
 * Copyright © 2018 Emu Analytics
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import Axios from 'axios';
import Debug from 'debug';
import { ApiKeyEndpoints } from './apikey.endpoints';
import { AppConfigEndpoints } from './appconfig.endpoints';
import { ApplicationEndpoints } from './application.endpoints';
import { AuthEndpoints } from './auth.edpoints';
import { DatabaseEndpoints } from './database.endpoints';
import { DatasetEndpoints } from './dataset.endpoints';
import { FlowEngineError } from './errors';
import { JobEndpoints } from './job.endpoints';
import { MapEndpoints } from './map.endoints';
import { MapLayerEndpoints } from './maplayer.endpoints';
import { PipelineEndpoints } from './pipeline.endpoints';
import { StyleEndpoints } from './style.endpoint';
import { TileSourceEndpoints } from './tile-source.endpoints';
import { TileEndpoints } from './tile.endpoints';
import { UploadEndpoints } from './upload.endpoints';
import { WebSocketGateway } from './web-socket-gateway';
var Client = /** @class */ (function () {
    function Client(baseUrl, apiKey, options) {
        if (apiKey === void 0) { apiKey = null; }
        this.baseUrl = baseUrl;
        this.apiKey = apiKey;
        this.debug = Debug('flow-engine-client');
        this.accessToken = null;
        this.auth = new AuthEndpoints(this);
        this.tiles = new TileEndpoints(this);
        this.maps = new MapEndpoints(this);
        this.apiKeys = new ApiKeyEndpoints(this);
        this.applications = new ApplicationEndpoints(this);
        this.appconfig = new AppConfigEndpoints(this);
        this.databases = new DatabaseEndpoints(this);
        this.datasets = new DatasetEndpoints(this);
        this.styles = new StyleEndpoints(this);
        this.tileSources = new TileSourceEndpoints(this);
        this.uploads = new UploadEndpoints(this);
        this.jobs = new JobEndpoints(this);
        this.maplayers = new MapLayerEndpoints(this);
        this.pipelines = new PipelineEndpoints(this);
        this.axios = Axios.create({
            baseURL: baseUrl,
            maxContentLength: 524288000,
            transformResponse: [
                function transformResponse(data, headers) {
                    var _a;
                    if ((_a = headers === null || headers === void 0 ? void 0 : headers['content-type']) === null || _a === void 0 ? void 0 : _a.startsWith('application/json')) {
                        // Force parsing of data if expected content type is JSON
                        // This ensures that errors are thrown if JSON is truncated due to a query timeout
                        // (default Axios behavior is simply to return a string if JSON parsing fails)
                        return JSON.parse(data);
                    }
                    return data;
                }
            ]
        });
        this.webSocketGateway = new WebSocketGateway(this, options);
    }
    Client.prototype.connect = function () {
        return this.webSocketGateway.connect();
    };
    Client.prototype.disconnect = function () {
        return this.webSocketGateway.disconnect();
    };
    Client.prototype.config = function () {
        return this._get('/api/v1/config');
    };
    Client.prototype._get = function (path, options) {
        var _a;
        if (options === void 0) { options = {}; }
        // Move apiKey from params
        if ((_a = options.params) === null || _a === void 0 ? void 0 : _a.apiKey) {
            options.apiKey = options.params.apiKey;
            delete options.params.apiKey;
        }
        return this.axios(__assign(__assign({ method: 'get', url: path }, options), this.headers(options)))
            .then(function (res) { return res.data; })
            .catch(this.handleError);
    };
    Client.prototype._post = function (path, data, options) {
        var _a;
        if (options === void 0) { options = {}; }
        // Move apiKey from params
        if ((_a = options.params) === null || _a === void 0 ? void 0 : _a.apiKey) {
            options.apiKey = options.params.apiKey;
            delete options.params.apiKey;
        }
        return this.axios(__assign(__assign({ method: 'post', url: path, data: data, maxContentLength: Infinity, maxBodyLength: Infinity }, options), this.headers(options)))
            .then(function (res) { return res.data; })
            .catch(this.handleError);
    };
    Client.prototype._put = function (path, data, options) {
        var _a;
        if (options === void 0) { options = {}; }
        // Move apiKey from params
        if ((_a = options.params) === null || _a === void 0 ? void 0 : _a.apiKey) {
            options.apiKey = options.params.apiKey;
            delete options.params.apiKey;
        }
        return this.axios(__assign(__assign({ method: 'put', url: path, data: data }, options), this.headers(options)))
            .then(function (res) { return res.data; })
            .catch(this.handleError);
    };
    Client.prototype._delete = function (path, options) {
        var _a;
        if (options === void 0) { options = {}; }
        // Move apiKey from params
        if ((_a = options.params) === null || _a === void 0 ? void 0 : _a.apiKey) {
            options.apiKey = options.params.apiKey;
            delete options.params.apiKey;
        }
        return this.axios(__assign(__assign({ method: 'delete', url: path }, options), this.headers(options)))
            .then(function (res) { return res.data; })
            .catch(this.handleError);
    };
    // Subscribe to named channel - errors and messages are delivered via the callback
    Client.prototype._subscribe = function (channel, cb) {
        if (!this.webSocketGateway.isConnected) {
            this.webSocketGateway.connect();
        }
        return this.webSocketGateway.subscribe(channel, cb);
    };
    // Unsubscribe from named channel - optional callback is called on completion
    Client.prototype._unsubscribe = function (channel, cb) {
        if (!this.webSocketGateway.isConnected) {
            this.webSocketGateway.connect();
        }
        this.webSocketGateway.unsubscribe(channel, cb);
    };
    /**
     * Normalize errors
     * Throws a new error based on type of input error
     *
     * @param {AxiosError} e
     */
    Client.prototype.handleError = function (e) {
        // Flow API error
        if (e.response && e.response.data) {
            if (typeof e.response.data === 'object') {
                var data = e.response.data;
                throw new FlowEngineError(data.message || data.error, data.error, e.response.status);
            }
            else if (typeof e.response.data === 'string') {
                throw new FlowEngineError(e.response.data, null, e.response.status);
            }
        }
        // Generic error
        // Don't leak information in error
        var cleanError = e;
        delete cleanError.config;
        delete cleanError.request;
        delete cleanError.response;
        throw new FlowEngineError(cleanError.message, cleanError);
    };
    Client.prototype.headers = function (options) {
        var headers = options.headers || {};
        if (options.apiKey || this.apiKey) {
            headers['X-Api-Key'] = options.apiKey || this.apiKey;
        }
        // Add default auth header if token supplied and not already set in options
        if (this.accessToken) {
            headers.Authorization = headers.Authorization || "Bearer ".concat(this.accessToken);
        }
        return { headers: headers };
    };
    return Client;
}());
export { Client };
//# sourceMappingURL=client.js.map